﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        public void demoMemorizedEntries()
        {
            // A Memorized Entry holds a combination of group, project, codes, and comment which is
            // shown as a row on the user's time, expense, or mileage entry screen.  For this demo
            // we'll create one using some information that we know is in the database.
            journyx.MemorizedEntry mem_entry = new journyx.MemorizedEntry {
                group = "Any",
                project = "root",
                code1 = "Coding",
                code2 = "Regular",
                code3 = "Unbillable",
                comment = "Test memorized entry"
            };
            string mentry_id = api.createUserMemorizedEntry(session_key, "time", USER, mem_entry);
            Console.WriteLine("... memorized entry ID: {0}", mentry_id);

            mem_entry.code1 = "Dev";
            string mentry_id2 = api.modifyUserMemorizedEntry(session_key, "time", mentry_id, mem_entry);
            Debug.Assert(mentry_id == mentry_id2);

            journyx.MemorizedEntry[] mem_entries = api.getUserMemorizedEntries(session_key, "time", USER);
            foreach (journyx.MemorizedEntry entry in mem_entries) {
                Console.WriteLine("... memorized entry for {0}: {1} {2}/{3}/{4}/{5}/{6}",
                    USER, entry.id, entry.project, entry.code1, entry.code2, entry.code3, entry.comment);
            }

            api.deleteUserMemorizedEntry(session_key, "time", mentry_id);

            // Groups can have memorized entries too.
            journyx.MemorizedEntry g_mem_entry = new journyx.MemorizedEntry {
                group = testgroup_id,
                project = "root",
                code1 = "Coding",
                code2 = "Regular",
                code3 = "Unbillable",
                comment = "Test group entry"
            };
            string g_mentry_id = api.createGroupMemorizedEntry(session_key, "time", testgroup_id, g_mem_entry);
            Console.WriteLine("... group memorized entry ID: {0}", g_mentry_id);

            g_mem_entry.code1 = "Dev";
            string g_mentry_id2 = api.modifyGroupMemorizedEntry(session_key, "time", g_mentry_id, g_mem_entry);
            Debug.Assert(g_mentry_id == g_mentry_id2);

            journyx.MemorizedEntry[] g_mem_entries = api.getGroupMemorizedEntries(session_key, "time", testgroup_id);
            foreach (journyx.MemorizedEntry entry in g_mem_entries) {
                Console.WriteLine("... group memorized entry for {0}: {1}", testgroup_id, entry.id);
            }

            api.deleteGroupMemorizedEntry(session_key, "time", g_mentry_id);
        }

        public void demoJournals()
        {
            // Journals can be connected to projects or to specific entries.  First, let's create a test
            // project and demonstrate journals using it.
            string project_id = api.addFullProject(session_key, new journyx.ProjectRecord { id = "jtest00", name = "jtest00", parent = "root", creator = USER });

            // Create a new journal.  Note that if you call createJournalForProject again with the same
            // journal name, it will return the ID of the existing journal instead of making a new one.
            string jid = api.createJournalForProject(session_key, project_id, "testjournal0");
            Console.WriteLine("journal created: {0}", jid);

            string[] journal_ids = api.getJournalsForProject(session_key, project_id);
            foreach (string j in journal_ids) {
                Console.WriteLine("... journal id in project {0}: {1}", project_id, j);
            }
 
            // Now add some notes.  The note text can be any length.
            api.createNote(session_key, jid, "test note one", "", "");
            api.createNote(session_key, jid, "test note two", "", "");

            // getNotesForJournal returns a set of note records.
            // The last three parameters (userid, startdate, enddate) can be filled in to get only
            // a matching subset of notes.
            journyx.NoteRecord[] noterecs = api.getNotesForJournal(session_key, jid, "", "", "");
            foreach (journyx.NoteRecord nrec in noterecs) {
                Console.WriteLine("... note: {0} {1}", nrec.id, nrec.the_text);
            }

            // If we know the id of a note, we can modify the text.  Note that the first note
            // we created is actually the second one in the journal, because creating the journal
            // automatically creates an initial note to serve as a creation timestamp.
            api.modifyNoteText(session_key, noterecs[1].id, "modified test note one");

            // Once you modify a note, the ids of the underlying records may change, so you
            // cannot modify it again without re-fetching the records from the server.
            noterecs = api.getNotesForJournal(session_key, jid, "", "", "");

            // This time we'll clean up after ourselves while printing them.  Note that we
            // can't delete the initial note of the journal (the id of which is in note_rec_start
            // in each note record in the journal).
            foreach (journyx.NoteRecord nrec in noterecs) {
                Console.WriteLine("... modified notes: {0} {1}", nrec.id, nrec.the_text);
                if (nrec.id != nrec.note_rec_start) {
                    api.deleteNote(session_key, nrec.id);
                }
            }

            // Notes can also be attached to specific entries, in addition to being part
            // of a particular journal.  Here, we demonstrate with a time entry.
            journyx.TimeRecord trec = api.getDefaultTimeRecord(session_key);
            trec.hours = 2.0;
            trec.date = "20110801";
            trec.project = project_id;
            string trec_id = api.addFullTimeRecord(session_key, trec);

            api.createNote(session_key, jid, "test time record note", "time", trec_id);

            // Now we can fetch the note text by the id of the entry it is attached to.
            // The "separator" argument (here an empty string) is placed between notes in
            // the output text, if there is more than one.
            string note = api.getEntryNoteById(session_key, "time", trec_id, "");
            Console.WriteLine("time entry note: {0}", note);

            // Clean up.
            api.removeTimeRecord(session_key, trec_id);
            api.deleteJournal(session_key, jid);
            api.removeProject(session_key, project_id);
        }

        public void demoAccruals()
        {
            // Managing accruals is relatively simple: jxAPI methods exist to create,
            // modify, get, and delete them.

            journyx.AccrualRecord acc_rec = new journyx.AccrualRecord {
                code = "Holiday",
                type = "Soft",
                hours = 10.0
            };
            string acc_id = api.createUserAccrual(session_key, testuser_id, acc_rec);
            Console.WriteLine("accrual created with id {0}", acc_id);

            acc_rec.hours = 24.0;
            string acc_id2 = api.modifyUserAccrual(session_key, acc_id, acc_rec);
            Debug.Assert(acc_id == acc_id2);

            journyx.AccrualRecord[] accruals = api.getUserAccruals(session_key, testuser_id);
            foreach (journyx.AccrualRecord acc in accruals) {
                Console.WriteLine("... accrual {0}: {1} {2} {3}", acc.id, acc.code, acc.type, acc.hours);
            }

            api.deleteUserAccrual(session_key, acc_id);
        }
    }
}
