﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace JournyxAPISample
{
    partial class Program
    {
        public void demoJXCoreTypes()
        {
            demoTimeRecords();
            demoExpenseRecords();
            demoGroups();
            demoUsers();
            demoCodes();
            demoSubcodes();
            demoSubsubcodes();
            demoProjects();
            demoSheets();
        }

        //
        // The jxAPI offers both type-specific and generic record manipulation methods.
        // The type-specific methods support basic operations on the primary database
        // tables, including the following:
        //
        // - getDefault<Type>Record, which returns a new record partially filled with
        //   default values.  Always use this when constructing a new record, because
        //   some fields may be validated on the server side even if you do not intend
        //   to alter them.
        //
        // - addFull<Type>Record adds a record to the database and returns its
        //   server-assigned ID.
        //
        // - get<Type>Record accepts a partially filled record acting as a pattern,
        //   and returns the first record matching it.
        //
        // - modify<Type>Record updates a record with new contents.
        //
        // - remove<Type>Record removes a record specified by its ID.
        //
        // The generic database methods expand on these operations and can be used with
        // any Journyx table supported by the jxAPI, but at the cost of additional
        // complexity.  The following functions demonstrate basic operations on the core
        // jxAPI types, as well as additional type-specific jxAPI methods where
        // available; the generic methods are demonstrated separately, in jxAPIGenericDB.cs.

        public void demoTimeRecords()
        {
            journyx.TimeRecord trec;

            // Obtain an "empty" record with required defaults already filled in.            
            trec = api.getDefaultTimeRecord(session_key);

            // A default time record includes the user ID so it is not necessary
            // to set it here, unless you are an admin who is setting a time record
            // for a different user.
            trec.date = jxTodaysDate();
            trec.hours = 2.0;
            trec.comment = "time record test one";

            // Add the record we just modified to the database.  
            string trec_id = api.addFullTimeRecord(session_key, trec);

            Console.WriteLine("... added time record with id {0}", trec_id);

            // Retrieve the record we just added using its ID.
            // Note that the get<Type>Record methods take a pattern, which is
            // a blank object with only the relevant matching fields filled in,
            // and returns the first matching record.
            journyx.TimeRecord trecPattern = new journyx.TimeRecord { id = trec_id };
            journyx.TimeRecord trec1 = api.getTimeRecord(session_key, trecPattern);

            Console.WriteLine("... retrieved record with id {0} ({1}, {2}, {3})", trec_id, trec1.user, trec1.hours, trec1.comment);

            // Now modify the record.  The modify<Type>Record method updates an 
            // existing record (specified by its ID) with new contents.
            trec1.date = jxTodaysDate();
            trec1.hours = 3.5;
            trec1.comment = "time record test two";

            api.modifyTimeRecord(session_key, trec_id, trec1);

            trec1 = api.getTimeRecord(session_key, trecPattern);
            Console.WriteLine("... retrieved modified record with id {0} ({1}, {2})", trec_id, trec1.hours, trec1.comment);

            // Because updating the hours in a time record is likely to be a pretty common
            // operation, there is a special jxAPI method for it.
            api.updateTimeValue(session_key, trec_id, 5.0);

            trec1 = api.getTimeRecord(session_key, trecPattern);
            Console.WriteLine("... record {0} hours updated to {1}", trec_id, trec1.hours);

            // Finally, delete the sample record we've been working with.
            api.removeTimeRecord(session_key, trec_id);
        }

        public void demoExpenseRecords()
        {
            // Create, retrieve, and modify an expense record using the basic methods.
            journyx.ExpenseRecord erec = api.getDefaultExpenseRecord(session_key);

            erec.amount = 20.00;
            erec.currency = "US$";
            erec.expense_code = "Miscellaneous";
            erec.source = "Employee";
            erec.date = jxTodaysDate();

            string erec_id = api.addFullExpenseRecord(session_key, erec);

            journyx.ExpenseRecord erecPattern = new journyx.ExpenseRecord { id = erec_id };
            journyx.ExpenseRecord erec1 = api.getExpenseRecord(session_key, erecPattern);
            Debug.Assert(erec_id.Equals(erec1.id));

            erec1.amount = 30.00;
            erec1.comment = "test expense record";
            api.modifyExpenseRecord(session_key, erec1.id, erec1);

            erec = api.getExpenseRecord(session_key, erecPattern);
            Debug.Assert(erec1.id.Equals(erec.id) && erec1.comment.Equals(erec.comment));

            // While time record codes, subcodes, and subsubcodes have their own
            // jxAPI methods, expense codes, currencies, and sources are stored separately
            // so there are some convenience functions to list them.  For historical
            // reasons, the returned array is alternating pairs of codes and printable
            // names for them.
            string[] exp_codes = api.getExpenseCodeList(session_key);
            Console.WriteLine("... valid expense codes: {0}", formatList(exp_codes));

            string[] exp_currencies = api.getExpenseCurrencyList(session_key);
            Console.WriteLine("... valid currencies: {0}", formatList(exp_currencies));

            string[] exp_sources = api.getExpenseSourceList(session_key);
            Console.WriteLine("... valid expense sources: {0}", formatList(exp_sources));

            // As with time records, there is a quick way to modify the amount of an
            // expense record.
            api.updateExpenseValue(session_key, erec_id, 40.00);

            erec = api.getExpenseRecord(session_key, erecPattern);
            Console.WriteLine("... new expense record amount: {0}", erec.amount);

            // Delete the sample record.
            api.removeExpenseRecord(session_key, erec_id);
        }


        //  Groups can be manipulated like other core record types, but have additional methods
        //  to handle other objects' membership in them.
        //
        public void demoGroups()
        {
            journyx.GroupRecord grec;

            grec = api.getDefaultGroup(session_key);
            // Unlike some other object types, you can assign your own group IDs.
            grec.id = "testgrp1";
            grec.name = "TestGroup1";
            grec.description = "Test Group 1";

            string grec_id = api.addFullGroup(session_key, grec);
            journyx.GroupRecord grecPattern = new journyx.GroupRecord { id = grec_id };
            journyx.GroupRecord grec1 = api.getGroup(session_key, grecPattern);
            Debug.Assert(grec_id.Equals(grec1.id));

            grec1.description = "Test Group 1 Description";
            api.modifyGroup(session_key, grec_id, grec1);
            grec = api.getGroup(session_key, grecPattern);
            Debug.Assert(grec1.description.Equals(grec.description));

            // For the core types that can be part of a group, there are
            // add<Type>ToGroup and remove<Type>FromGroup methods which will be
            // demonstrated below as part of the sample code for those types.
            // For object types that are not part of the jxAPI core set, the
            // following generic functions can be used to manipulate group
            // membership.  Note that for these functions, the group and object
            // IDs may be either an actual ID, or the group/object name.

            // Note that a "task" is actually just a "code" internally so we could have
            // used addCodeToGroup/removeCodeFromGroup here, but this is just a demo
            // of how these methods work.
            api.addObjectToGroup(session_key, grec_id, "task", "Dev");
            // listGroupObjects returns ids of objects in a group...
            Console.WriteLine("... tasks in {0}: {1}", grec_id, formatList(api.listGroupObjects(session_key, grec_id, "codes")));
            // .. and listObjectGroups lists IDs of groups an object is in.
            Console.WriteLine("... groups task Dev is in: {0}", formatList(api.listObjectGroups(session_key, "task", "Dev")));
            api.removeObjectFromGroup(session_key, grec_id, "task", "Dev");

            api.removeGroup(session_key, grec_id);
        }

        public void demoUsers()
        {
            journyx.UserRecord urec;

            urec = api.getDefaultUser(session_key);
            urec.user_login = "testuser";
            urec.full_name = "Test Tester";
            // GUI values are not filled in the default user record.  You must
            // pass something here; the values shown here are defaults for a
            // fresh Timesheet install, but you may wish to use something more
            // appropriate if you are adding users from a jxAPI application.
            urec.time_gui = "Standard Time Entry Grid";
            urec.expense_gui = "Standard Expense Entry Grid";
            urec.mileage_gui = "Standard Mileage Entry Grid";

            string user_id = api.addFullUser(session_key, urec);
            journyx.UserRecord userPattern = new journyx.UserRecord { id = user_id };
            journyx.UserRecord urec1 = api.getUser(session_key, userPattern);
            Debug.Assert(urec1.user_login.Equals(urec.user_login));

            // Older versions of Timesheet treated the id field of user records as
            // the login.  This is no longer always the case, so to find a user by
            // login, compare the user_login field instead.
            urec1 = api.getUser(session_key, new journyx.UserRecord { user_login = USER });
            Console.WriteLine("... installing user: {0} {1}", urec1.user_login, urec1.full_name);

            urec1.full_name = "Test Tester Jr.";
            api.modifyUser(session_key, urec1.id, urec1);
            urec = api.getUser(session_key, new journyx.UserRecord { id = urec1.id });
            Debug.Assert(urec.id.Equals(urec1.id));

            // Users can be part of groups.
            api.addUserToGroup(session_key, user_id, testgroup_id);
            Console.WriteLine("... user in group {0}: {1}", testgroup_id, formatList(api.listGroupObjects(session_key, testgroup_id, "user")));
            api.removeUserFromGroup(session_key, user_id, testgroup_id);

            // Users have roles, which determine what their permissions are.
            // The installing user, for example, is an administrator.
            string[] roles = api.getUserRoles(session_key, USER);
            Console.WriteLine("... user {0} has roles {1}", USER, formatList(roles));

            // You can assign and remove roles from users through the jxAPI.
            api.assignRoleToUser(session_key, user_id, "Project Manager");
            Console.WriteLine("... user {0} now has roles {1}", user_id, formatList(api.getUserRoles(session_key, user_id)));
            api.removeRoleFromUser(session_key, user_id, "Project Manager");

            api.removeUser(session_key, user_id);

            // Also see the login() method in Program.cs for how to change a user's
            // password.
        }

        // In a fresh install of Timesheet, codes, subcodes, and subsubcodes are
        // called "tasks", "pay types", and "bill types" respectively, and can be
        // renamed by the administrator.  Regardless of how they are labeled in the
        // application, the jxAPI uses the same set of structures and methods to
        // work with them.
        public void demoCodes()
        {
            journyx.CodeRecord crec;

            // Elsewhere in this demo, you may see codes being added without
            // using getDefaultCode.  While it is true that in the current
            // version of the jxAPI a code record only needs an id and pretty_name,
            // you should not rely on that for production code.
            crec = api.getDefaultCode(session_key);
            crec.id = "testcode0";
            crec.pretty_name = "TestCode0";
            crec.description = "Test Code 0";

            string code_id = api.addFullCode(session_key, crec);
            journyx.CodeRecord codePattern = new journyx.CodeRecord { id = code_id };
            journyx.CodeRecord crec1 = api.getCode(session_key, codePattern);
            Debug.Assert(code_id.Equals(crec1.id));

            crec1.description = "Test Code 0 Description";
            api.modifyCode(session_key, crec1.id, crec1);
            crec = api.getCode(session_key, codePattern);
            Debug.Assert(crec.id.Equals(crec1.id));

            api.addCodeToGroup(session_key, code_id, testgroup_id);
            api.removeCodeFromGroup(session_key, code_id, testgroup_id);

            // The getCodeList method returns a list of alternating id, name pairs
            // for all codes in the database.
            string[] codes = api.getCodeList(session_key);
            Console.WriteLine("... all codes: {0}", formatList(codes));

            api.removeCode(session_key, code_id);
        }

        // Subcodes and Subsubcodes are exactly like Codes except for their type
        // and method names, but in order to exercise the jxAPI they are shown here
        // anyway.
        public void demoSubcodes()
        {
            journyx.SubcodeRecord crec;

            // Elsewhere in this demo, you may see codes being added without
            // using getDefaultCode.  While it is true that in the current
            // version of the jxAPI a code record only needs an id and pretty_name,
            // you should not rely on that for production code.
            crec = api.getDefaultSubcode(session_key);
            crec.id = "testsubcode0";
            crec.pretty_name = "TestSubCode0";
            crec.description = "Test SubCode 0";

            string code_id = api.addFullSubcode(session_key, crec);
            journyx.SubcodeRecord codePattern = new journyx.SubcodeRecord { id = code_id };
            journyx.SubcodeRecord crec1 = api.getSubcode(session_key, codePattern);
            Debug.Assert(code_id.Equals(crec1.id));

            crec1.description = "Test SubCode 0 Description";
            api.modifySubcode(session_key, crec1.id, crec1);
            crec = api.getSubcode(session_key, codePattern);
            Debug.Assert(crec.id.Equals(crec1.id));

            api.addSubcodeToGroup(session_key, code_id, testgroup_id);
            api.removeSubcodeFromGroup(session_key, code_id, testgroup_id);

            // The getCodeList method returns a list of alternating id, name pairs
            // for all codes in the database.
            string[] codes = api.getSubcodeList(session_key);
            Console.WriteLine("... all codes: {0}", formatList(codes));

            api.removeSubcode(session_key, code_id);
        }

        public void demoSubsubcodes()
        {
            journyx.SubsubcodeRecord crec;

            // Elsewhere in this demo, you may see codes being added without
            // using getDefaultCode.  While it is true that in the current
            // version of the jxAPI a code record only needs an id and pretty_name,
            // you should not rely on that for production code.
            crec = api.getDefaultSubsubcode(session_key);
            crec.id = "testsubsubcode0";
            crec.pretty_name = "TestSubsubCode0";
            crec.description = "Test SubsubCode 0";

            string code_id = api.addFullSubsubcode(session_key, crec);
            journyx.SubsubcodeRecord codePattern = new journyx.SubsubcodeRecord { id = code_id };
            journyx.SubsubcodeRecord crec1 = api.getSubsubcode(session_key, codePattern);
            Debug.Assert(code_id.Equals(crec1.id));

            crec1.description = "Test SubCode 0 Description";
            api.modifySubsubcode(session_key, crec1.id, crec1);
            crec = api.getSubsubcode(session_key, codePattern);
            Debug.Assert(crec.id.Equals(crec1.id));

            api.addSubsubcodeToGroup(session_key, code_id, testgroup_id);
            api.removeSubsubcodeFromGroup(session_key, code_id, testgroup_id);

            // The getCodeList method returns a list of alternating id, name pairs
            // for all codes in the database.
            string[] codes = api.getSubsubcodeList(session_key);
            Console.WriteLine("... all codes: {0}", formatList(codes));

            api.removeSubsubcode(session_key, code_id);
        }

        public void demoProjects()
        {
            journyx.ProjectRecord prec;

            prec = api.getDefaultProject(session_key);
            prec.id = "testproject0";
            prec.name = "TestProject0";
            prec.description = "Test Project 0";
            prec.parent = "root";  // "root" is the ID of the root of the project tree.
            prec.creator = USER;   // Not filled in by default and may not be left empty.

            string project_id = api.addFullProject(session_key, prec);
            journyx.ProjectRecord projectPattern = new journyx.ProjectRecord { id = project_id };
            journyx.ProjectRecord prec1 = api.getProject(session_key, projectPattern);
            Debug.Assert(project_id.Equals(prec1.id));

            prec1.description = "Test Project 0 Description";
            api.modifyProject(session_key, prec1.id, prec1);
            prec = api.getProject(session_key, projectPattern);
            Debug.Assert(prec.id.Equals(prec1.id));

            api.addProjectToGroup(session_key, project_id, testgroup_id);
            Console.WriteLine("... projects in group {0}: {1}", testgroup_id, formatList(api.listGroupObjects(session_key, testgroup_id, "project")));
            api.removeProjectFromGroup(session_key, project_id, testgroup_id);

            // Project dependencies on code/subcode/subsubcodes can also be manipulated.
            string code_id = api.addFullCode(session_key, new journyx.CodeRecord { id = "depcode0", pretty_name = "depcode0" });
            string subcode_id = api.addFullSubcode(session_key, new journyx.SubcodeRecord { id = "depsc0", pretty_name = "depsc0" });
            string subsubcode_id = api.addFullSubsubcode(session_key, new journyx.SubsubcodeRecord { id = "depssc0", pretty_name = "depssc0" });

            api.addProjectDependency(session_key, project_id, "code", code_id);
            api.addProjectDependency(session_key, project_id, "subcode", subcode_id);
            api.addProjectDependency(session_key, project_id, "subsubcode", subsubcode_id);

            // Note that this always returns CodeRecords (which are actually just like SubcodeRecords and SubsubcodeRecords).
            journyx.CodeRecord[] deps = api.getProjectDependencies(session_key, project_id, "subcode");
            foreach (journyx.CodeRecord crec in deps) {
                Console.WriteLine("... project subcode dependency: {0}", crec.id);
            }

            api.removeProjectDependency(session_key, project_id, "subsubcode", subsubcode_id);
            api.removeProjectDependency(session_key, project_id, "subcode", subcode_id);
            api.removeProjectDependency(session_key, project_id, "code", code_id);

            api.removeCode(session_key, code_id);
            api.removeSubcode(session_key, subcode_id);
            api.removeSubsubcode(session_key, subsubcode_id);
            api.removeProject(session_key, project_id);
        }

        // Sheets are not represented by a client-side record type, but the jxAPI has methods
        // to manipulate them by ID.
        public void demoSheets()
        {
            // Sheets are created automatically as needed.  The type can be "time", "expense" or "mileage".
            string today_sheet_id = api.getSheetIDByDate(session_key, "time", jxTodaysDate());
            Console.WriteLine("today's timesheet ID:       {0}", today_sheet_id);
            Console.WriteLine("latest sheet ID:            {0}", api.getLatestSheetID(session_key, "time"));
            Console.WriteLine("next sheet ID:              {0}", api.getNextSheetID(session_key, "time", today_sheet_id));
            Console.WriteLine("previous sheet ID:          {0}", api.getPreviousSheetID(session_key, "time", today_sheet_id));
            Console.WriteLine("dates in today's timesheet: {0}", formatList(api.getDatesInSheet(session_key, "time", today_sheet_id)));
            Console.WriteLine("today's sheet status:       {0}", api.getSheetStatus(session_key, "time", today_sheet_id));
            Console.WriteLine("most recent sheet statuses: {0}", formatList(api.getRecentSheetStatus(session_key, "time", 2)));
            Console.WriteLine("period dates for some date: {0}", formatList(api.getPeriodDates(session_key, "time", "20110620")));
            // The following are the possible statuses which mean a sheet has been rejected.
            Console.WriteLine("rejected statuses:          {0}", formatList(api.getSheetRejectedStatuses(session_key, "time")));

            // Add a time record and examine some properties of its sheet.
            journyx.TimeRecord trec = api.getDefaultTimeRecord(session_key);
            trec.hours = 2.5;
            trec.date = "20110620";
            trec.comment = "test time record";
            string trec_id = api.addFullTimeRecord(session_key, trec);
            string sheet_id = api.getSheetIDByDate(session_key, "time", "20110620");

            Console.WriteLine("total hours in sheet:       {0}", api.getTotalHoursInTimeSheet(session_key, sheet_id));
            Console.WriteLine("record IDs in sheet:        {0}", api.getRecordIDsInSheet(session_key, "time", sheet_id));

            api.removeTimeRecord(session_key, trec_id);
        }
    }
}