package com.journyx.jxapidemo;

import jxapi.*;
import com.journyx.jxapi.*;

import java.rmi.RemoteException;
import static java.lang.System.out;

public class jxapi_core_demo {

    static jxapi_demo app;
    static String session_key;
    static JxapiStub api;

    public jxapi_core_demo(jxapi_demo demoapp){
        app = demoapp;    
        session_key = jxapi_demo.session_key;
        api = jxapi_demo.api;
    }

    public void run() throws RemoteException {
        demoTimeRecords();
        demoExpenseRecords();
        demoGroups();
        demoUsers();
        demoCodes();
        demoSubcodes();
        demoSubsubcodes();
        demoProjects();
        demoSheets();
    }
    
    // The jxAPI offers both type-specific and generic record manipulation methods.
    // The type-specific methods support basic operations on the primary database
    // tables, including the following:
    //
    // - getDefault<Type>Record, which returns a new record partially filled with
    //   default values.  Always use this when constructing a new record, because
    //   some fields may be validated on the server side even if you do not intend
    //   to alter them.
    //
    // - addFull<Type>Record adds a record to the database and returns its
    //   server-assigned ID.
    //
    // - get<Type>Record accepts a partially filled record acting as a pattern,
    //   and returns the first record matching it.
    //
    // - modify<Type>Record updates a record with new contents.
    //
    // - remove<Type>Record removes a record specified by its ID.
    //
    // The generic database methods expand on these operations and can be used with
    // any Journyx table supported by the jxAPI, but at the cost of additional
    // complexity.  The following functions demonstrate basic operations on the core
    // jxAPI types, as well as additional type-specific jxAPI methods where
    // available.

    public void demoTimeRecords() throws RemoteException {
        TimeRecord trec;

        // Obtain an "empty" time record with required defaults already filled in.
        trec = api.getDefaultTimeRecord(session_key);

        // A default time record includes the user ID, so it is not necessary to
        // set it unless you are an admin who is adding records for a different user.
        trec.setDate("20110622");
        trec.setHours(2.0);
        trec.setComment("Time record test");

        // Add the record to the database.
        String trec_id = api.addFullTimeRecord(session_key, trec);
        out.format("... added time record with id %s\n", trec_id);

        // Retrieve the record we just added using its ID.  The get<Type>Record
        // methods take a pattern, which is a record that is totally empty except
        // for the relevant search fields filled in.  Simply instantiating a
        // TimeRecord object for the pattern doesn't work however, since its
        // string attributes will have null values which will cause an exception
        // when passed through SOAP.  The jxapi_demo.newRecordWithID method is
        // used here as a convenience; it creates a new record from the specified
        // class, then sets all of its string fields to "" so that it can pass
        // through SOAP, then sets the id field for this common case of looking
        // up a record by its ID.
        TimeRecord trec_pattern = (TimeRecord)app.newRecordWithID(TimeRecord.class, trec_id);
        TimeRecord trec1 = api.getTimeRecord(session_key, trec_pattern);
        out.format("... retrieved record %s (%s, %s, %s)\n", trec_id, trec1.getUser(), trec1.getHours(), trec1.getComment());

        // Now modify the record.  The modify<Type>Record method updates an
        // existing record (specified by its ID) with new contents.
        trec1.setDate("20110623");
        trec1.setHours(3.5);
        trec1.setComment("time record test two");

        api.modifyTimeRecord(session_key, trec_id, trec1);

        // Check that it has been modified.
        trec1 = api.getTimeRecord(session_key, trec_pattern);
        out.format("... retrieved modified record %s (%s, %s, %s)\n",
                trec_id, trec1.getUser(), trec1.getHours(), trec1.getComment());

        // Because updating the hours in a time record is likely to be a common
        // operation, there is a special method for it.
        api.updateTimeValue(session_key, trec_id, 5.0);

        trec1 = api.getTimeRecord(session_key, trec_pattern);
        out.format("... updated record %s hours to %s\n", trec_id, trec1.getHours());

        // Finally, delete the sample record we have been working with.
        api.removeTimeRecord(session_key, trec_id);

    }

    // Expense records are quite similar to time records, with different names for
    // the code/subcode/subsubcode fields and an amount instead of hours.
    public void demoExpenseRecords() throws RemoteException {

        ExpenseRecord erec = api.getDefaultExpenseRecord(session_key);
        erec.setAmount(20.00);
        erec.setCurrency("US$");
        erec.setExpense_code("Miscellaneous");
        erec.setSource("Employee");
        erec.setDate("20110622");

        String erec_id = api.addFullExpenseRecord(session_key, erec);

        ExpenseRecord erec_pattern = (ExpenseRecord)app.newRecordWithID(ExpenseRecord.class, erec_id);
        ExpenseRecord erec1 = api.getExpenseRecord(session_key, erec_pattern);
        assert erec_id.equals(erec1.getId());

        erec1.setAmount(30.00);
        erec1.setComment("test expense record");
        api.modifyExpenseRecord(session_key, erec_id, erec1);

        erec = api.getExpenseRecord(session_key, erec_pattern);
        assert erec.getId().equals(erec1.getId()) && erec.getComment().equals(erec1.getComment());

        // While time record codes/subcodes/subsubcodes have their own jxAPI methods,
        // expense codes/currencies/sources are stored in different tables so there
        // are some convenience functions to list them.  For historical reasons, the
        // returned array is alternating pairs of codes and printable names for them.
        String[] exp_codes = api.getExpenseCodeList(session_key);
        out.format("... valid expense codes: %s\n", app.formatList(exp_codes));

        String[] exp_currencies = api.getExpenseCurrencyList(session_key);
        out.format("... valid expense currencies: %s\n", app.formatList(exp_currencies));

        String[] exp_sources = api.getExpenseSourceList(session_key);
        out.format("... valid expense sources: %s\n", app.formatList(exp_sources));

        // As with time records, there is a quick way to modify the amount of an expense record.
        api.updateExpenseValue(session_key, erec_id, 40.00);
        erec = api.getExpenseRecord(session_key, erec_pattern);
        out.format("... new expense record amount: %s\n", erec.getAmount());

        api.removeExpenseRecord(session_key, erec_id);

    }

    // Groups are manipulated like other core method types, but have additional methods
    // to handle other objects' membership in them.  Additionally, objects that can be
    // members of groups have associated methods which will be demonstrated in the following
    // sections.
    public void demoGroups() throws RemoteException {
        GroupRecord grec = api.getDefaultGroup(session_key);

        // Unlike some other object types, you can assign your own group IDs.
        grec.setId("testgrp1");
        grec.setName("TestGroup1");
        grec.setDescription("Test Group 1");

        String grec_id = api.addFullGroup(session_key, grec);
        GroupRecord grec_pattern = (GroupRecord) app.newRecordWithID(GroupRecord.class, grec_id);
        GroupRecord grec1 = api.getGroup(session_key, grec_pattern);
        assert grec_id.equals(grec1.getId());

        grec1.setDescription("test group 1 description");
        api.modifyGroup(session_key, grec_id, grec1);
        grec = api.getGroup(session_key, grec_pattern);
        assert grec1.getDescription().equals(grec.getDescription());

        // For the core types that can be in a group, there are add<Type>ToGroup
        // and remove<Type>FromGroup methods which will be demonstrated below.
        // For other object types that are not part of the jxAPI core, the
        // following generic methods can be used to manipulate group membership.
        // Note that for these methods the group and object IDs may be either an
        // actual ID or the group/object name.

        // Note that a "task" is actually just a Code internally, so we could have
        // used addCodeToGroup/removeCodeFromGroup here, but this is just a demo
        // of how these methods work.
        api.addObjectToGroup(session_key, grec_id, "task", "Dev");

        // listGroupObjects returns a list of IDs of objects in the group.
        String[] grp_objs = api.listGroupObjects(session_key, grec_id, "code");
        out.format("... tasks in %s: %s\n", grec_id, app.formatList(grp_objs));

        // and listObjectGroups lists IDs of groups an object is in.
        String[] obj_grps = api.listObjectGroups(session_key, "task", "Dev");
        out.format("... groups task Dev is in: %s\n", app.formatList(obj_grps));

        api.removeObjectFromGroup(session_key, grec_id, "task", "Dev");
        api.removeGroup(session_key, grec_id);
    }

    public void demoUsers() throws RemoteException {
        UserRecord urec = api.getDefaultUser(session_key);
        urec.setUser_login("testuser");
        urec.setFull_name("Todd Tester");
        // As of API 8.0 the GUI defaults are not filled in the default
        // user record, and the jxAPI will reject the record if a valid GUI
        // name is not supplied.
        urec.setTime_gui("Standard Time Entry Grid");
        urec.setExpense_gui("Standard Expense Entry Grid");
        urec.setMileage_gui("Standard Mileage Entry Grid");

        String user_id = api.addFullUser(session_key, urec);
        UserRecord user_pattern = (UserRecord) app.newRecordWithID(UserRecord.class, user_id);
        UserRecord urec1 = api.getUser(session_key, user_pattern);
        assert urec1.getUser_login().equals(urec.getUser_login());

        // Older versions of Timesheet treated the id field of user records
        // as the login.  This is no longer always the case, so to find a user by
        // login, search on the user_login field instead.
        UserRecord user_pattern2  = (UserRecord) app.newRecord(UserRecord.class);
        user_pattern2.setUser_login(jxapi_demo.USER);
        urec = api.getUser(session_key, user_pattern2);
        out.format("... installing user: %s %s\n", urec.getUser_login(), urec.getFull_name());

        urec1.setFull_name("Theo Tester Jr.");
        api.modifyUser(session_key, urec1.getId(), urec1);
        urec = api.getUser(session_key, user_pattern);
        assert urec.getId().equals(urec1.getId());

        api.addUserToGroup(session_key, user_id, jxapi_demo.testgroup_id);
        api.removeUserFromGroup(session_key, user_id, jxapi_demo.testgroup_id);

        // Users have roles, which determine what their permissions are.
        // The installing user, for example, is an administrator.
        String roles[] = api.getUserRoles(session_key, jxapi_demo.USER);
        out.format("... user %s has roles %s\n", jxapi_demo.USER, app.formatList(roles));

        api.assignRoleToUser(session_key, user_id, "Project Manager");
        out.format("... user %s now has roles %s\n", 
                user_id, app.formatList(api.getUserRoles(session_key, user_id)));
        api.removeRoleFromUser(session_key, user_id, "Project Manager");

        api.removeUser(session_key, user_id);
    }

    // In a fresh install of Timesheet, codes, subcodes, and subsubcodes are called
    // "tasks", "pay types", and "bill types" respectively, and can be renamed by
    // the administrator.  Regardless of how they are labeled in the application,
    // the jxAPI uses the same set of structures and methods to work with them.

    public void demoCodes() throws RemoteException {
        CodeRecord crec = api.getDefaultCode(session_key);
        crec.setId("testcode0");
        crec.setPretty_name("TestCode0");
        crec.setDescription("Test Code 0");

        String code_id = api.addFullCode(session_key, crec);
        CodeRecord code_pattern = (CodeRecord) app.newRecordWithID(CodeRecord.class, code_id);
        CodeRecord crec1 = api.getCode(session_key, code_pattern);
        assert code_id.equals(crec1.getId());

        crec1.setDescription("Test code 0 description");
        api.modifyCode(session_key, crec1.getId(), crec1);
        crec = api.getCode(session_key, code_pattern);
        assert crec.getId().equals(crec1.getId());

        api.addCodeToGroup(session_key, code_id, jxapi_demo.testgroup_id);
        api.removeCodeFromGroup(session_key, code_id, jxapi_demo.testgroup_id);

        // The getCodeList method returns a list of alternating id, name pairs for all
        // codes in the database.
        String[] codes = api.getCodeList(session_key);
        out.format("... all codes: %s\n", app.formatList(codes));

        api.removeCode(session_key, code_id);
    }

    // Subcodes and Subsubcodes are exactly like Codes except for their type and
    // method names.
    public void demoSubcodes() throws RemoteException {
        SubcodeRecord crec = api.getDefaultSubcode(session_key);
        crec.setId("testsc0");
        crec.setPretty_name("TestSubcode0");
        crec.setDescription("Test Subcode 0");

        String code_id = api.addFullSubcode(session_key, crec);
        SubcodeRecord code_pattern = (SubcodeRecord) app.newRecordWithID(SubcodeRecord.class, code_id);
        SubcodeRecord crec1 = api.getSubcode(session_key, code_pattern);
        assert code_id.equals(crec1.getId());

        crec1.setDescription("Test subcode 0 description");
        api.modifySubcode(session_key, crec1.getId(), crec1);
        crec = api.getSubcode(session_key, code_pattern);
        assert crec.getId().equals(crec1.getId());

        api.addSubcodeToGroup(session_key, code_id, jxapi_demo.testgroup_id);
        api.removeSubcodeFromGroup(session_key, code_id, jxapi_demo.testgroup_id);

        // The getCodeList method returns a list of alternating id, name pairs for all
        // codes in the database.
        String[] codes = api.getSubcodeList(session_key);
        out.format("... all subcodes: %s\n", app.formatList(codes));

        api.removeSubcode(session_key, code_id);
    }

    public void demoSubsubcodes() throws RemoteException {
        SubsubcodeRecord crec = api.getDefaultSubsubcode(session_key);
        crec.setId("testssc0");
        crec.setPretty_name("TestSubsubcode0");
        crec.setDescription("Test Subsubcode 0");

        String code_id = api.addFullSubsubcode(session_key, crec);
        SubsubcodeRecord code_pattern = (SubsubcodeRecord) app.newRecordWithID(SubsubcodeRecord.class, code_id);
        SubsubcodeRecord crec1 = api.getSubsubcode(session_key, code_pattern);
        assert code_id.equals(crec1.getId());

        crec1.setDescription("Test subsubcode 0 description");
        api.modifySubsubcode(session_key, crec1.getId(), crec1);
        crec = api.getSubsubcode(session_key, code_pattern);
        assert crec.getId().equals(crec1.getId());

        api.addSubsubcodeToGroup(session_key, code_id, jxapi_demo.testgroup_id);
        api.removeSubsubcodeFromGroup(session_key, code_id, jxapi_demo.testgroup_id);

        // The getCodeList method returns a list of alternating id, name pairs for all
        // codes in the database.
        String[] codes = api.getSubsubcodeList(session_key);
        out.format("... all subsubcodes: %s\n", app.formatList(codes));

        api.removeSubsubcode(session_key, code_id);
    }

    public void demoProjects() throws RemoteException {
        ProjectRecord prec = api.getDefaultProject(session_key);
        prec.setId("testproject0");
        prec.setName("TestProject0");
        prec.setDescription("Test project 0");
        prec.setParent("root");  // "root" is the ID of the root of the project tree
        prec.setCreator(jxapi_demo.USER);

        String project_id = api.addFullProject(session_key, prec);
        ProjectRecord project_pattern = (ProjectRecord) app.newRecordWithID(ProjectRecord.class, project_id);
        ProjectRecord prec1 = api.getProject(session_key, project_pattern);
        assert project_id.equals(prec1.getId());

        prec1.setDescription("Test project 0 description");
        api.modifyProject(session_key, prec1.getId(), prec1);
        prec = api.getProject(session_key, project_pattern);
        assert prec.getId().equals(prec1.getId());

        api.addProjectToGroup(session_key, project_id, jxapi_demo.testgroup_id);
        api.removeProjectFromGroup(session_key, project_id, jxapi_demo.testgroup_id);

        // Project dependencies on codes/subcodes/subsubcodes can be manipulated as well.
        String code_id = api.addCode(session_key, "depcode0");
        String subcode_id = api.addSubcode(session_key, "depsc0");
        String subsubcode_id = api.addSubsubcode(session_key, "depssc0");

        api.addProjectDependency(session_key, project_id, "code", code_id);
        api.addProjectDependency(session_key, project_id, "subcode", subcode_id);
        api.addProjectDependency(session_key, project_id, "subsubcode", subsubcode_id);

        // Note that getProjectDependencies always returns CodeRecords, even if you ask
        // for subcodes or subsubcodes.
        CodeRecord[] deps = api.getProjectDependencies(session_key, project_id, "subcode");
        
        for (CodeRecord crec : deps) {
            out.format("... project subcode dependency: %s\n", crec.getId());
        }

        api.removeProjectDependency(session_key, project_id, "subsubcode", subsubcode_id);
        api.removeProjectDependency(session_key, project_id, "subcode", subcode_id);
        api.removeProjectDependency(session_key, project_id, "code", code_id);

        api.removeCode(session_key, code_id);
        api.removeSubcode(session_key, subcode_id);
        api.removeSubsubcode(session_key, subsubcode_id);
        api.removeProject(session_key, project_id);
    }

    public void demoSheets() throws RemoteException {
        // Sheets are automatically created as needed.  The type can be "time",
        // "expense", or "mileage".

        String sheet_id = api.getSheetIDByDate(session_key, "time", "20110622");
        out.format("sheet id for 2011/06/22:     %s\n", sheet_id);
        out.format("latest sheet id:             %s\n", api.getLatestSheetID(session_key, "time"));
        out.format("next sheet id:               %s\n", api.getNextSheetID(session_key, "time", sheet_id));
        out.format("previous sheet id:           %s\n", api.getPreviousSheetID(session_key, "time", sheet_id));
        out.format("dates in this sheet:         %s\n",
                app.formatList(api.getDatesInSheet(session_key, "time", sheet_id)));
        out.format("this sheet status:           %s\n", api.getSheetStatus(session_key, "time", sheet_id));
        out.format("most recent statuses:        %s\n",
                app.formatList(api.getRecentSheetStatus(session_key, "time", 2)));
        out.format("period dates for 2011/01/01: %s\n",
                app.formatList(api.getPeriodDates(session_key, "time", "20110101")));
        out.format("possible reject statuses:    %s\n",
                app.formatList(api.getSheetRejectedStatuses(session_key, "time")));

        // Add a time record and examine some properties of its sheet.
        TimeRecord trec = api.getDefaultTimeRecord(session_key);
        trec.setHours(2.5);
        trec.setDate("20110622");
        trec.setComment("sheet test record");
        String trec_id = api.addFullTimeRecord(session_key, trec);
        sheet_id = api.getSheetIDByDate(session_key, "time", trec.getDate());

        out.format("total hours in sheet: %s\n", api.getTotalHoursInTimeSheet(session_key, sheet_id));
        out.format("record IDs in sheet: %s\n",
                app.formatList(api.getRecordIDsInSheet(session_key, "time", sheet_id)));

        api.removeTimeRecord(session_key, trec_id);
    }
}

